
/////////////////////////////////////////////////////////////////////////////////////
// HUD.cpp
/////////////////////////////////////////////////////////////////////////////////////

#include "nds.h"
#include <stdio.h>

#include "Viewport.h"
#include "SpriteManager.h"
#include "HUD.h"

/////////////////////////////////////////////////////////////////////////////////////
// variables
/////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////
// HUD()
/////////////////////////////////////////////////////////////////////////////////////

HUD::HUD(void) {

}

/////////////////////////////////////////////////////////////////////////////////////
// ~HUD()
/////////////////////////////////////////////////////////////////////////////////////

HUD::~HUD(void) {

}

/////////////////////////////////////////////////////////////////////////////////////
// drawTimeValue() - draws a time value (MM:SS:??)
/////////////////////////////////////////////////////////////////////////////////////

static const int timeDivisorTable[] = {
	10*(60*1000),
	1*(60*1000),
	10*1000,
	1*1000,
	100,
	10
};

void HUD::drawTimeValue(volatile unsigned short *vram, SpriteManager *spriteManager, Viewport *viewport, int time, int tX, int tY) {

	// max at 59:59:99
	if (time > 59*(60*1000) + (59*1000) + 999)
		time = 59*(60*1000) + (59*1000) + 999;

	int n = 0;
	for (int j = 0; j < 6; j++) {
		if (n == 2) {
			// :
			n = 0;
			spriteManager->drawSprite(vram, SPRITE_ID_TWO_DOTS, tX, tY, 0, viewport);
			tX += 4;
		}

		// the value
		int v = time / timeDivisorTable[j];
		spriteManager->drawSprite(vram, SPRITE_ID_NUMBER_0 + v, tX, tY, 0, viewport);
		tX += 4;
		time -= v * timeDivisorTable[j];
		n++;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// draw() - draw the HUD
/////////////////////////////////////////////////////////////////////////////////////

void HUD::draw(volatile unsigned short *vram, SpriteManager *spriteManager, Viewport *viewport, int lap, int lapTimeCurrent, int lapTimeBest, int speedPercentage) {

	int hudY = viewport->startY + 20;
	int hudX = viewport->startX;

	// LAP
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_L, hudX +  4, hudY, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_A, hudX +  8, hudY, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_P, hudX + 12, hudY, 0, viewport);

	if (lap > 9)
		lap = 9;
	spriteManager->drawSprite(vram, SPRITE_ID_NUMBER_0 + lap, hudX + 20, hudY, 0, viewport);

	// LAP TIME
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_L, hudX +  4, hudY + 8, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_A, hudX +  8, hudY + 8, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_P, hudX + 12, hudY + 8, 0, viewport);

	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_T, hudX + 20, hudY + 8, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_I, hudX + 24, hudY + 8, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_M, hudX + 28, hudY + 8, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_E, hudX + 32, hudY + 8, 0, viewport);

	drawTimeValue(vram, spriteManager, viewport, lapTimeCurrent, hudX + 40, hudY + 8);

	// BEST LAP
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_B, hudX +  4, hudY + 16, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_E, hudX +  8, hudY + 16, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_S, hudX + 12, hudY + 16, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_T, hudX + 16, hudY + 16, 0, viewport);

	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_L, hudX + 24, hudY + 16, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_A, hudX + 28, hudY + 16, 0, viewport);
	spriteManager->drawSprite(vram, SPRITE_ID_LETTER_P, hudX + 32, hudY + 16, 0, viewport);

	drawTimeValue(vram, spriteManager, viewport, lapTimeBest, hudX + 40, hudY + 16);

	// TACHOMETER
	int dX = viewport->dX*40/100;
	int dY = 8;
	hudX = viewport->startX + viewport->dX - dX - 2;

	// back
	int color = (SPRITE_COLOR_TACHO_BG << 8) | SPRITE_COLOR_TACHO_BG;
	for (int fY = 0; fY < dY; fY++) {
		volatile unsigned short *ptr16 = &vram[((hudY + fY) << 7) + hudX];
		for (int k = 0; k < dX; k++)
			*(ptr16++) = color;
	}

	// front (speedPercentage is [0, 256])
	dX = ((dX - 2) * speedPercentage) >> 8;
	if (dX > 0) {
		dY -= 2;
		hudX++;
		int color = (SPRITE_COLOR_TACHO_FG << 8) | SPRITE_COLOR_TACHO_FG;
		for (int fY = 0; fY < dY; fY++) {
			volatile unsigned short *ptr16 = &vram[((hudY + fY + 1) << 7) + hudX];
			for (int k = 0; k < dX; k++)
				*(ptr16++) = color;
		}
	}
}
