
/////////////////////////////////////////////////////////////////////////////////////
// SpriteManager.cpp
/////////////////////////////////////////////////////////////////////////////////////

#include "nds.h"
#include <stdio.h>

#include "Viewport.h"
#include "SpriteManager.h"
#include "inflate.h"

/////////////////////////////////////////////////////////////////////////////////////
// internal variables
/////////////////////////////////////////////////////////////////////////////////////

// the sprite coordinate sheet
// TODO: load also this from a file
static const int spriteCoordinateSheet[] = {
	 0,  0, 48, 40, // PLAYER_CAR_FRAME_1
	 0, 64, 48, 40, // PLAYER_CAR_FRAME_2
	 0*8+64, 0*8, 8, 8, // LETTER_A
	 1*8+64, 0*8, 8, 8, // LETTER_B
	 2*8+64, 0*8, 8, 8, // LETTER_C
	 3*8+64, 0*8, 8, 8, // LETTER_D
	 4*8+64, 0*8, 8, 8, // LETTER_E
	 5*8+64, 0*8, 8, 8, // LETTER_F
	 6*8+64, 0*8, 8, 8, // LETTER_G
	 7*8+64, 0*8, 8, 8, // LETTER_H
	 0*8+64, 1*8, 8, 8, // LETTER_I
	 1*8+64, 1*8, 8, 8, // LETTER_J
	 2*8+64, 1*8, 8, 8, // LETTER_K
	 3*8+64, 1*8, 8, 8, // LETTER_L
	 4*8+64, 1*8, 8, 8, // LETTER_M
	 5*8+64, 1*8, 8, 8, // LETTER_N
	 6*8+64, 1*8, 8, 8, // LETTER_O
	 7*8+64, 1*8, 8, 8, // LETTER_P
	 0*8+64, 2*8, 8, 8, // LETTER_Q
	 1*8+64, 2*8, 8, 8, // LETTER_R
	 2*8+64, 2*8, 8, 8, // LETTER_S
	 3*8+64, 2*8, 8, 8, // LETTER_T
	 4*8+64, 2*8, 8, 8, // LETTER_U
	 5*8+64, 2*8, 8, 8, // LETTER_V
	 6*8+64, 2*8, 8, 8, // LETTER_W
	 7*8+64, 2*8, 8, 8, // LETTER_X
	 0*8+64, 3*8, 8, 8, // LETTER_Y
	 1*8+64, 3*8, 8, 8, // LETTER_Z
	 3*8+64, 3*8, 8, 8, // NUMBER_0
	 4*8+64, 3*8, 8, 8, // NUMBER_1
	 5*8+64, 3*8, 8, 8, // NUMBER_2
	 6*8+64, 3*8, 8, 8, // NUMBER_3
	 7*8+64, 3*8, 8, 8, // NUMBER_4
	 0*8+64, 4*8, 8, 8, // NUMBER_5
	 1*8+64, 4*8, 8, 8, // NUMBER_6
	 2*8+64, 4*8, 8, 8, // NUMBER_7
	 3*8+64, 4*8, 8, 8, // NUMBER_8
	 4*8+64, 4*8, 8, 8, // NUMBER_9
	 0*8+64, 5*8, 8, 8, // TWO_DOTS
	 128,  0,  8, 12, // ROADSIDE_AD_1
	 136,  0, 12, 18, // ROADSIDE_AD_2
	 148,  0, 18, 27, // ROADSIDE_AD_3
	 166,  0, 28, 42, // ROADSIDE_AD_4
	  64, 64, 42, 63, // ROADSIDE_AD_5
	 128, 64, 64, 96, // ROADSIDE_AD_6
	 0, 227, 128, 20 // BACKGROUND_CITY
};

/////////////////////////////////////////////////////////////////////////////////////
// SpriteManager()
/////////////////////////////////////////////////////////////////////////////////////

SpriteManager::SpriteManager(void) {

	spriteData = NULL;
}

/////////////////////////////////////////////////////////////////////////////////////
// ~SpriteManager()
/////////////////////////////////////////////////////////////////////////////////////

SpriteManager::~SpriteManager(void) {

	if (spriteData != NULL)
		delete[] spriteData;
}

/////////////////////////////////////////////////////////////////////////////////////
// spriteSheetFree() - free the current sprite sheet
/////////////////////////////////////////////////////////////////////////////////////

void SpriteManager::spriteSheetFree(void) {

	if (spriteData != NULL) {
		delete[] spriteData;
		spriteData = NULL;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// spriteSheetLoad() - actually uncompresses the specified sprite sheet
/////////////////////////////////////////////////////////////////////////////////////

void SpriteManager::spriteSheetLoad(unsigned char *compressedSheet, unsigned short *palette) {

	// allocate room for the sprites
	if (spriteData == NULL)
		spriteData = new unsigned short[128*256];

	// uncompress the image data
	inflate(compressedSheet, spriteData);

	// load in the palette
	for (int i = 0; i < 256; i++)
		BG_PALETTE_SUB[i] = palette[i];

	// overwrite some colors
	// TODO: add these to the palette file itself, don't do here
	BG_PALETTE_SUB[SPRITE_COLOR_SKY]         = RGB15(98 >> 3, 147 >> 3, 231 >> 3);
	BG_PALETTE_SUB[SPRITE_COLOR_GROUND]      = RGB15(94 >> 3, 131 >> 3, 62 >> 3);
	BG_PALETTE_SUB[SPRITE_COLOR_ROAD_1]      = RGB15(64 >> 3, 64 >> 3, 64 >> 3);
	BG_PALETTE_SUB[SPRITE_COLOR_ROAD_2]      = RGB15(48 >> 3, 48 >> 3, 48 >> 3);
	BG_PALETTE_SUB[SPRITE_COLOR_TACHO_BG]    = RGB15(75 >> 3, 113 >> 3, 176 >> 3);
	BG_PALETTE_SUB[SPRITE_COLOR_TACHO_FG]    = RGB15(158 >> 3, 185 >> 3, 229 >> 3);
	BG_PALETTE_SUB[SPRITE_COLOR_FINISH_LINE] = RGB15(255 >> 3, 255 >> 3, 255 >> 3);
}

/////////////////////////////////////////////////////////////////////////////////////
// getSpriteWidth() - does just that
/////////////////////////////////////////////////////////////////////////////////////

int SpriteManager::getSpriteWidth(int spriteID) {

	// TODO: inline this?
	return spriteCoordinateSheet[(spriteID << 2) + 2];
}

/////////////////////////////////////////////////////////////////////////////////////
// drawSprite() - draws the specified sprite
/////////////////////////////////////////////////////////////////////////////////////

void SpriteManager::drawSprite(volatile unsigned short *vram, int spriteID, int sX, int sY, int flags, Viewport *viewport) {

	// get sprite information
	spriteID <<= 2;

	int pX = spriteCoordinateSheet[spriteID + 0];
	int pY = spriteCoordinateSheet[spriteID + 1];
	int dX = spriteCoordinateSheet[spriteID + 2];
	int dY = spriteCoordinateSheet[spriteID + 3];

	// vram bus is 16bit wide, and access to our sprite graphics is behind a 16bit pointer as well
	dX >>= 1;
	pX >>= 1;

	// ... so we lose some horizontal resolution, but doesn't matter in this demo

	// center the sprite?
	if (flags & SPRITE_CENTERED_X)
		sX -= dX >> 1;
	if (flags & SPRITE_CENTERED_Y)
		sY -= dY >> 1;
	else if (flags & SPRITE_PIVOT_BOTTOM)
		sY -= dY;

	// is the sprite outside the viewport?
	if (sX > viewport->endX || sY > viewport->endY || sX+dX < viewport->startX || sY+dY < viewport->startY)
		return;

	// draw the sprite
	// TODO: this is pretty slow, but fortunately this "prototype" is not about drawing sprites fast so
	// let's keep this simple
	for (int tY = 0; tY < dY; tY++) {
		for (int tX = 0; tX < dX; tX++) {
			// transparency test (color 0 is transparent in all our palettes, thanks to our toolchain)
			int in = spriteData[((pY+tY) << 7) + (pX+tX)];
			if (in == 0)
				continue;

			// clip (also slow)
			int cY = sY+tY;
			if (cY < viewport->startY || sY >= viewport->endY)
				continue;
			int cX = sX+tX;
			if (cX < viewport->startX || cX >= viewport->endX)
				continue;

			// draw two pixels
			vram[(cY << 7) + cX] = in;
		}
	}
}
