
/////////////////////////////////////////////////////////////////////////////////////
// main.cpp
/////////////////////////////////////////////////////////////////////////////////////

#include "nds.h"
#include <stdio.h>

#include "main.h"
#include "Viewport.h"
#include "Car.h"
#include "SpriteManager.h"
#include "Road.h"
#include "HUD.h"
#include "GameManager.h"

/////////////////////////////////////////////////////////////////////////////////////
// global variables
/////////////////////////////////////////////////////////////////////////////////////

// the main mode for the program
int mainMode = MAIN_MODE_GAME_INIT;

// the game manager
GameManager *gameManager = NULL;

// the keys, current and past
int keysCurrent = 0;
int keysOld = 0;

// the screen size
int screenDX = 256;
int screenDY = 192;

/////////////////////////////////////////////////////////////////////////////////////
// main() - the program main
/////////////////////////////////////////////////////////////////////////////////////

int main(void) {

	// init DS
	powerON(POWER_ALL);

	// enable vbi
	irqInit();
	irqEnable(IRQ_VBLANK);

	while (1) {
		// read the keys
		scanKeys();

		// get the keys
		keysCurrent = keysHeld();

		switch(mainMode) {
		case MAIN_MODE_GAME_INIT:
			// init the gameManager
			if (gameManager == NULL)
				gameManager = new GameManager();
			gameManager->init();
			gameManager->precalculateScreen(screenDX, screenDY);
			gameManager->reset();

			// all inits are done, move to the game
			mainMode = MAIN_MODE_GAME;

			break;

		case MAIN_MODE_GAME:
			// DS runs everything 60fps, if the program is fast enough
			// and this game is...
			gameManager->draw();
			gameManager->iteration(1000/60, keysCurrent, keysOld);

			// reshape the view?
#ifdef DEBUG
			if ((keysCurrent & KEY_L) != 0 && (keysOld & KEY_L) == 0) {
				if (screenDX > 160) {
					screenDX -= 32;
					screenDY -= 24;
				}
				gameManager->precalculateScreen(screenDX, screenDY);
			}
			if ((keysCurrent & KEY_R) != 0 && (keysOld & KEY_R) == 0) {
				if (screenDX < 256) {
					screenDX += 32;
					screenDY += 24;
				}
				gameManager->precalculateScreen(screenDX, screenDY);
			}
#endif

			break;
		}

		// remember the key statuses
		keysOld = keysCurrent;

		// wait for vertical blank
		swiWaitForVBlank();
	}

	return 0;
}
